@@ -1,7 +1,7 @@
-import logging
 import time
 from uuid import uuid4
 import requests
+from .utils import logger
 
 REQUEST_TIMEOUT = 20
 
@@ -10,7 +10,7 @@ class _StatsigNetwork:
 
     __RETRY_CODES = [408, 500, 502, 503, 504, 522, 524, 599]
 
-    def __init__(self, sdkKey, options):
+    def __init__(self, sdkKey, options, error_boundary):
         self.__sdk_key = sdkKey
         api = options.api
         if not options.api.endswith("/"):
@@ -18,10 +18,11 @@ class _StatsigNetwork:
         self.__api = api
         self.__timeout = options.timeout or REQUEST_TIMEOUT
         self.__local_mode = options.local_mode
-        self.__log = logging.getLogger('statsig.sdk')
+        self.__error_boundary = error_boundary
+        self.__log = logger
         self.__session = str(uuid4())
 
-    def post_request(self, endpoint, payload):
+    def post_request(self, endpoint, payload, log_on_exception = False):
         if self.__local_mode:
             self.__log.debug('Using local mode. Dropping network request')
             return None
@@ -39,14 +40,16 @@ class _StatsigNetwork:
                 data = response.json()
                 if data:
                     return data
-                else:
-                    return None
+                return None
+            return None
         except Exception as e:
-            self.__log.warning(
-                'Network exception caught when making request to %s failed', endpoint)
+            if log_on_exception:
+                self.__error_boundary.log_exception(e)
+                self.__log.warning(
+                    'Network exception caught when making request to %s failed', endpoint)
             return None
 
-    def retryable_request(self, endpoint, payload):
+    def retryable_request(self, endpoint, payload, log_on_exception = False):
         if self.__local_mode:
             return None
 
@@ -61,16 +64,18 @@ class _StatsigNetwork:
                 self.__api + endpoint, json=payload, headers=headers, timeout=self.__timeout)
             if response.status_code in self.__RETRY_CODES:
                 return payload
-            elif response.status_code >= 300:
+            if response.status_code >= 300:
                 self.__log.warning(
                     "Request to %s failed with code %d", endpoint, response.status_code)
             return None
         except Exception as e:
-            self.__log.warning(
-                "Network exception caught when making request to %s failed", endpoint)
+            if log_on_exception:
+                self.__error_boundary.log_exception(e)
+                self.__log.warning(
+                    "Network exception caught when making request to %s failed", endpoint)
             return None
 
-    def get_request(self, url, headers):
+    def get_request(self, url, headers, log_on_exception = False):
         if self.__local_mode:
             return None
         try:
@@ -79,9 +84,10 @@ class _StatsigNetwork:
                 url, headers=headers, timeout=self.__timeout)
             if response.ok:
                 return response
-            else:
-                return None
+            return None
         except Exception as e:
-            self.__log.warning(
-                'Network exception caught when making request to %s failed', url)
+            if log_on_exception:
+                self.__error_boundary.log_exception(e)
+                self.__log.warning(
+                    'Network exception caught when making request to %s failed', url)
             return None
